@@ -45,10 +45,21 @@ def test_exceptions():
     assert "unknown" in s2
 
 
+def test_buggy_builtin_repr():
+    # Simulate a case where a repr for a builtin raises.
+    # reprlib dispatches by type name, so use "int".
+
+    class int:
+        def __repr__(self):
+            raise ValueError("Buggy repr!")
+
+    assert "Buggy" in saferepr(int())
+
+
 def test_big_repr():
     from _pytest._io.saferepr import SafeRepr
 
-    assert len(saferepr(range(1000))) <= len("[" + SafeRepr().maxlist * "1000" + "]")
+    assert len(saferepr(range(1000))) <= len("[" + SafeRepr(0).maxlist * "1000" + "]")
 
 
 def test_repr_on_newstyle():
